{% load allauth %}
{% setvar variant %}
    {% if "warning" in attrs.tags %}
        warning
    {% elif "danger" in attrs.tags %}
        danger
    {% elif "secondary" in attrs.tags %}
        secondary
    {% elif "success" in attrs.tags %}
        success
    {% else %}
        primary
    {% endif %}
{% endsetvar %}
<span {% if attrs.title %}title="{{ attrs.title }}"{% endif %}
      class="badge text-bg-{{ variant }}">
    {% slot %}
    {% endslot %}
</span>
