{% extends "allauth/layouts/manage.html" %}
{% load allauth %}
{% block head_title %}Profile{% endblock %}
{% block content %}
    {% element h1 %}
        Profile
    {% endelement %}
    <p>Your profile</p>
{% endblock content %}
