{% load allauth %}
<li class="list-group-item">
    <a class="d-block w-100 link-dark link-underline link-underline-opacity-0"
       title="{{ attrs.name }}"
       href="{{ attrs.href }}">{{ attrs.name }}</a>
</li>
